@@ -437,6 +437,7 @@ public:
 	bool						objcGc() const { return fObjCGc; }
 	bool						objcGcOnly() const { return fObjCGcOnly; }
 	bool						canUseThreadLocalVariables() const { return fTLVSupport; }
+	bool						shouldUseBuildVersion(ld::Platform plat, uint32_t minOSvers) const;
 	bool						addVersionLoadCommand() const { return fVersionLoadCommand && (platforms().count() != 0); }
 	bool						addFunctionStarts() const { return fFunctionStartsLoadCommand; }
 	bool						addDataInCodeInfo() const { return fDataInCodeInfoLoadCommand; }
@@ -500,6 +501,7 @@ public:
 	bool						needsSourceVersionLoadCommand() const { return fSourceVersionLoadCommand; }
 	bool						canUseAbsoluteSymbols() const { return fAbsoluteSymbols; }
 	bool						allowSimulatorToLinkWithMacOSX() const { return fAllowSimulatorToLinkWithMacOSX; }
+	bool						isSimulatorSupportDylib() const { return fSimulatorSupportDylib; }
 	uint64_t					sourceVersion() const { return fSourceVersion; }
 	uint32_t					sdkVersion() const { return fSDKVersion; }
 	const char*					demangleSymbol(const char* sym) const;
@@ -615,6 +617,7 @@ private:
 	void						addSegmentRename(const char* srcSegment, const char* dstSegment);
 	void						addSymbolMove(const char* dstSegment, const char* symbolList, std::vector<SymbolsMove>& list, const char* optionName);
 	void						cannotBeUsedWithBitcode(const char* arg);
+	bool 						sharedCacheEligiblePath(const char* path);
 
 
 //	ObjectFile::ReaderOptions			fReaderOptions;
@@ -769,6 +772,7 @@ private:
 	bool								fVersionLoadCommand;
 	bool								fVersionLoadCommandForcedOn;
 	bool								fVersionLoadCommandForcedOff;
+	bool								fForceLegacyVersionLoadCommands;
 	bool								fFunctionStartsLoadCommand;
 	bool								fFunctionStartsForcedOn;
 	bool								fFunctionStartsForcedOff;
@@ -786,6 +790,7 @@ private:
 	bool								fExportDynamic;
 	bool								fAbsoluteSymbols;
 	bool								fAllowSimulatorToLinkWithMacOSX;
+	bool								fSimulatorSupportDylib;
 	bool								fKeepDwarfUnwind;
 	bool								fKeepDwarfUnwindForcedOn;
 	bool								fKeepDwarfUnwindForcedOff;
