@@ -89,6 +89,20 @@ extern "C" double log2 ( double );
 #include "parsers/lto_file.h"
 #include "parsers/opaque_section_file.h"
 
+const ld::Platform ld::basePlatform(const ld::Platform& platform)  {
+	switch(platform) {
+		case ld::kPlatform_iOSMac:
+		case ld::kPlatform_iOSSimulator:
+			return ld::kPlatform_iOS;
+		case ld::kPlatform_watchOSSimulator:
+			return kPlatform_watchOS;
+		case ld::kPlatform_tvOSSimulator:
+			return ld::kPlatform_tvOS;
+		default:
+			return platform;
+	}
+}
+
 const ld::VersionSet ld::File::_platforms;
 
 struct PerformanceStatistics {
