@@ -14,6 +14,8 @@
 #if PROPERTIES
 	@property(readonly) int property1;
 	@property(readonly) int property2;
+	@property(class,readonly) int property3;
+	@property(class,readonly) int property4;
 #endif
 @end
 
@@ -24,6 +26,8 @@
 #if PROPERTIES
 	-(int) property1 { return 0; }
 	-(int) property2 { return 0; }
+	+(int) property3 { return 0; }
+	+(int) property4 { return 0; }
 #endif
 @end
 
